(**********************************************************************)
(*                                                                    *)
(*                           Moca                                     *)
(*                                                                    *)
(*          Pierre Weis, INRIA Rocquencourt                           *)
(*          Frdric Blanqui, projet Protheo, INRIA Lorraine          *)
(*                                                                    *)
(*  Copyright 2005-2008,                                              *)
(*  Institut National de Recherche en Informatique et en Automatique. *)
(*  All rights reserved.                                              *)
(*                                                                    *)
(*  This file is distributed under the terms of the Q Public License. *)
(*                                                                    *)
(**********************************************************************)

(* result type for comparison functions *)

type ord =
  | Greater
  | Equal
  | Smaller
  | Uncomparable;;

type 'a cmp = 'a -> 'a -> ord;;

(* basic functions on ord *)

val int_of_ord : ord -> int;; (* undefined on Uncomparable *)

val opp : ord -> ord;;
val comp : ord -> ord -> ord;;

val ge : ord -> bool;;
val le : ord -> bool;;

(* lexicographic extension of an ordering *)

val lex : 'a cmp -> 'a list cmp;;

(* lexicographic path ordering *)

open Term;;
open Symb;;

exception IncompletePrecedence of symb * symb;;

val lpo_fail : symb cmp -> term cmp;;

val lpo : symb cmp -> term cmp;;
(* return Uncomparable on IncompletePrecedence *)

val mpo : symb cmp -> term cmp;;

val rpo : (Symb.symb, Parsetree.status) Hashtbl.t -> symb cmp -> term cmp;; 
